import base64
import os
import sqlite3

static_images = os.path.dirname(os.path.abspath(__file__)) + "/static/images/styles/small/"
# static_images = os.path.dirname(os.path.abspath(__file__)) + "/static/images/styles/small/"
save_parent_folder = "/storage/emulated/0/Music/EisenRadio"
script_path = os.path.dirname(__file__)


def convert_ascii(file_name):
    with open(file_name, "rb") as reader:
        img_bytes = reader.read()
        img_ascii = render_picture(img_bytes, 'encode')
    return img_ascii


def render_picture(byte_data, de_enc):
    render_pic = ''
    if de_enc == 'encode':
        render_pic = base64.b64encode(byte_data).decode('ascii')
    if de_enc == 'decode':
        render_pic = base64.b64decode(byte_data).decode('ascii')
    return render_pic


def make_db_from_schema(db_path):
    # # # # custom filled db # # #

    # connection = sqlite3.connect((os.path.join(script_path, 'database.db')))
    connection = sqlite3.connect(str(db_path))
    with open((os.path.join(script_path, 'schema.sql'))) as f:
        connection.executescript(f.read())

    cur = connection.cursor()
    cur.execute("INSERT INTO eisen_intern (browser_open,statistics,commercials) VALUES (?,?,?)", (str(1), 1, 1))

    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('classic',
         'http://37.251.146.169:8000/streamHD',
         save_parent_folder,
         'recontextualize next-generation vortals, engineer plug-and-play experiences, deploy dynamic action-items, '
         'synthesize transparent relationships, transition next-generation mindshare, morph dynamic paradigms, '
         'orchestrate end-to-end initiatives, incubate frictionless content, revolutionize efficient e-services, '
         'redefine impactful users, engage leading-edge eyeballs, envisioneer 24/365 deliverables, incentivize viral '
         'infomediaries, whiteboard clicks-and-mortar experiences, generate B2C e-commerce, envisioneer extensible '
         'convergence, morph 24/7 experiences, utilize customized e-services, strategize customized systems, '
         'scale proactive markets, exploit bleeding-edge partnerships, repurpose integrated infrastructures, '
         'seize cutting-edge relationships, brand enterprise e-services, morph value-added partnerships, '
         'recontextualize customized communities, enable magnetic web services, maximize cutting-edge relationships,'
         'integrate front-end synergies, embrace one-to-one supply-chains, leverage proactive functionalities',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-brown_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('deep_house',
         'https://streaming-01.xtservers.com:7000/stream',
         save_parent_folder,
         '',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-black_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('audio_noir',
         'http://104.192.169.54:8000/noir',
         save_parent_folder,
         'usa - Classic Old Time Radio, Sci Fi, Comedy, Drama',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-blue_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('anime_jp',
         'http://streamingv2.shoutcast.com/japanimradio-tokyo',
         save_parent_folder,
         'sound tracks of japanese anime videos and games',
         "image/jpeg",
         convert_ascii(static_images + "mixer-tempelhof-airport.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('radio_pan_america',
         'http://51.222.8.101:8000/stream',
         save_parent_folder,
         '',
         "image/jpeg",
         convert_ascii(static_images + "mixer-reload.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('YeahMon',
         'http://c3.radioboss.fm:8095/autodj',
         save_parent_folder,
         '',
         "image/jpeg",
         convert_ascii(static_images + "mixer-construction-site.jpg"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('playUrban',
         'http://live.playradio.org:9090/UrbanHD',
         save_parent_folder,
         '',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-red_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Nachtflug',
         'http://85.195.88.149:11810/sid=1',
         save_parent_folder,
         '',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-violet_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('hm',
         'https://hirschmilch.de:7001/prog-house.mp3',
         save_parent_folder,
         '',
         "image/png",
         convert_ascii(static_images + "radio3d-style-whiteneongelb_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('zenStyle',
         'https://radio4.cdm-radio.com:18004/stream-mp3-Zen',
         save_parent_folder,
         'recontextualize next-generation vortals, engineer plug-and-play experiences, deploy dynamic action-items, '
         'synthesize transparent relationships, transition next-generation mindshare, morph dynamic paradigms, '
         'orchestrate end-to-end initiatives, incubate frictionless content, revolutionize efficient e-services, '
         'redefine impactful users, engage leading-edge eyeballs, envisioneer 24/365 deliverables, incentivize viral '
         'infomediaries, whiteboard clicks-and-mortar experiences, generate B2C e-commerce, envisioneer extensible '
         'convergence, morph 24/7 experiences, utilize customized e-services, strategize customized systems, '
         'scale proactive markets, exploit bleeding-edge partnerships, repurpose integrated infrastructures, '
         'seize cutting-edge relationships, brand enterprise e-services, morph value-added partnerships, '
         'recontextualize customized communities, enable magnetic web services, maximize cutting-edge relationships,'
         'integrate front-end synergies, embrace one-to-one supply-chains, leverage proactive functionalities',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-white_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Paloma',
         'https://pool.radiopaloma.de/RADIOPALOMA.mp3',
         save_parent_folder,
         'Berlin /Germany',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-brown_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Bayern1',
         'https://streams.br.de/bayern1obb_2.m3u',
         save_parent_folder,
         '*m3u playlist server, redirect to first server in the list \n'
         'Bavaria / Germany \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-black_120x200.png"))
        )
    cur.execute(
        "INSERT INTO posts (title, content, download_path, pic_comment, pic_content_type, pic_data) VALUES (?, ?, ?, "
        "?, ?, ?)",
        ('Reggae',
         'http://hd.lagrosseradio.info:8000/lagrosseradio-reggae-192.mp3',
         save_parent_folder,
         'Paris / France \n\n'
         'recontextualize next-generation vortals  \n'
         'engineer plug-and-play experiences \n'
         'deploy dynamic action-items \n'
         'synthesize transparent relationships \n'
         'transition next-generation mindshare \n'
         'morph dynamic paradigms \n'
         'orchestrate end-to-end initiatives \n'
         'incubate frictionless content \n'
         'revolutionize efficient e-services \n'
         'redefine impactful users \n'
         'engage leading-edge eyeballs \n'
         'envisioneer 24/365 deliverables \n'
         'incentivize viral infomediaries \n'
         'generate B2C e-commerce \n'
         'envisioneer extensible convergence \n'
         'morph 24/7 experiences \n'
         'utilize customized e-services \n',
         "image/png",
         convert_ascii(static_images + "radio3d-style-white-neongreen_120x200.png"))
        )
    connection.commit()
    connection.close()


def empty_db_from_schema():
    # # # # custom filled db # # #
    connection = ''
    is_snap_device = 'SNAP' in os.environ
    is_android_device = 'ANDROID_STORAGE' in os.environ

    if is_android_device:
        return
    if not is_snap_device:
        connection = sqlite3.connect((str(os.path.join(script_path, 'database.db'))))
    if is_snap_device:
        connection = sqlite3.connect((str(os.path.join(os.environ["SNAP_USER_COMMON"], 'pre_configured.db'))))

    with open((os.path.join(script_path, 'schema.sql'))) as f:
        connection.executescript(f.read())

    cur = connection.cursor()
    cur.execute("INSERT INTO eisen_intern (browser_open,statistics,commercials) VALUES (?,?,?)", (str(1), 1, 1))
    connection.commit()
    connection.close()



import os
from flask import flash
from eisenradio.api import eisenApi


def make_folder(this_folder):
    rv = True
    if not os.path.exists(os.path.abspath(this_folder)):
        flash('Can create folder!\t' + os.path.abspath(this_folder), 'success')
        rv = True
    else:
        rv_write = eisen_write_test(os.path.abspath(this_folder))
        if not rv_write:
            flash('Folder exists, but write test failed!\t' + os.path.abspath(this_folder), 'danger')
            rv = False
            return rv
        if rv_write:
            rv = True

    if rv:
        try:
            os.makedirs(this_folder)
        except FileExistsError as err:
            if not err.errno == 17:    # pybug, py3.10, makedirs() ignores existing dirs
                print('-make_folder----------- FileExistsError' + str(err.errno))
                rv = False
        except PermissionError:
            print('-make_folder----------- PermissionError')
            rv = False
        except OSError:
            print('-make_folder----------- OSError')
            rv = False
    return rv


def eisen_write_test(write_file):
    test_file = os.path.join(write_file, 'eisen_test.txt')
    try:
        with open(test_file, 'wb') as record_file:
            record_file.write(b'\x03')
        os.remove(test_file)
    except OSError:  # master of desaster
        return False
    return True


def is_in_db_view(a_name):
    for radio_name in eisenApi.radio_id_name_dict.values():
        if radio_name.lower() == a_name.lower():
            return True
    return False


def remove_blank(a_name):
    rv = a_name.replace(" ", "")
    return rv


def remove_special_chars(str_name):
    str_name.replace(" ", "")
    # my_str = "hey th~!ere. /\ coolleagues?! Straße"
    rv = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\\|`~=+"""'})
    return rv


def radio_start_page_url_get(radio_name):
    """return the anchor point url with the port number of the server"""
    radio_id = ""
    for id_num, name in eisenApi.radio_id_name_dict.items():
        if name == radio_name:
            radio_id = id_num
    port = eisenApi.work_port
    radio_start_page_url = 'http://localhost:' + str(port) + '/#dot_' + str(radio_id)
    return radio_start_page_url


def radio_listen_button_get():
    """ return complete id of current pressed listen button"""
    l_list = [radio_id for radio_id, btn_down in eisenApi.lis_btn_dict.items() if btn_down]
    button_id = "Listen_" + str(l_list[0]) if len(l_list) > 0 else ""
    return button_id


def radio_name_get():
    """return name if id is in list else None"""
    l_list = [radio_id for radio_id, btn_down in eisenApi.lis_btn_dict.items() if btn_down]
    if len(l_list) > 0:
        radio_id = l_list[0]
        return eisenApi.radio_id_name_dict[radio_id]
    return


def parse_url_simple_url(radio_url):
    """called by routes.py, def post(post_id) to link to original radio URL """
    url = radio_url  # whole url is used for connection to radio server

    # 'http:' is first [0], 'ip:port/xxx/yyy' second item [1] in list_url_protocol
    list_url_protocol = url.split("//")
    list_url_ip_port = list_url_protocol[1].split("/")  # 'ip:port' is first item in list_url_ip_port
    radio_simple_url = list_url_protocol[0] + '//' + list_url_ip_port[0]
    return radio_simple_url



import threading
import time
from ghettorecorder import ghettoApi
from ghettorecorder import ghetto_recorder as ghetto
from eisenradio.api import eisenApi, eisenTest

radio_active = False
eisenApi.init_radio_active(radio_active)
thread_killer = (False, False, False)
eisenTest.init_test_thread_killer(thread_killer)


def start_watchdog_daemon():
    """path eishome.index_first_run()

    should never do any repair actions, only reporting (let it crash) so we know what to do next
    """
    threading.Thread(name="watchdog", target=run_watchdog, daemon=True).start()


def run_watchdog():
    while not ghettoApi.stop_blacklist_writer:
        # [print(f' {thread.name}') for thread in threading.enumerate()]
        report_active_radio_threads()
        # print_active_radio_threads()
        # report_failed_thread_names()    # testing
        # rv = "True, radio connected." if rv else "No radio connected."

        # print(f'\n\t blacklist_enabled {ghettoApi.blacklist_enable}\n')

        # [print(f'audio_stream_dict: {key}: {value}') for key, value in ghettoApi.ghetto_audio_stream_dict.items()]

        # [print(f'metadata_text: {key}: {value}') for key, value in ghettoApi.current_song_dict.items()]

        # [print(f'measure_dict: {key}: {value}') for key, value in ghettoApi.ghetto_measure_dict.items()]

        # for radio_name, blacklist in ghettoApi.all_blacklists_dict.items():
        #     print('\n')
        #     [print(f'bl: {radio_name[:3]}: {index} {value}') for index, value in enumerate(blacklist)]

        # [print(f'title_new: {key}: {value}') for key, value in ghettoApi.recorder_new_title_dict.items()]

        # disconnect browser to see filling and removal of queue items, buffer elements
        # [print(f'{key} queue len (max 4): {val.qsize()}') for key, val in ghettoApi.ghetto_audio_stream_dict.items()]

        # print(f"ghettoApi.all_blacklists_dict {ghettoApi.all_blacklists_dict}")

        for _ in range(10):
            # exit with the writer
            if ghettoApi.stop_blacklist_writer:
                break
            time.sleep(1)


def report_failed_thread_names():
    """write to error dict so js can write it to html

    For testing. If unlucky this guy can flood the html page with fake errors during station switching.
    A full crash of all radio threads at the same time can not be detected. omg
    rec. consists of three threads ["tail", "head", "meta"]
    meta - try to read metadata from radio
    head - clean up metadata string and build full path name for tail,
    tail - read/write the stream
    """
    record_desired_types = ["tail", "head", "meta"]
    listen_desired_types = ["tail", "meta"]

    all_threads_list = [thread.name for thread in threading.enumerate()]
    # classic_record_head, 12char 2nd part, (_record_head)  or (_listen_meta)
    active_rec_name_list = list(set([name[:len(name)-12] for name in all_threads_list if "_record_" in name]))
    active_lis_name_list = list(set([name[:len(name)-12] for name in all_threads_list if "_listen_" in name]))

    if len(active_rec_name_list) > 0:
        radio_rec_thread_dict = report_failed_thread_names_build_dict(active_rec_name_list, all_threads_list, "record")
        report_failed_thread_names_assert_dict(radio_rec_thread_dict, record_desired_types, "record")

    if len(active_lis_name_list) > 0:
        radio_lis_thread_dict = report_failed_thread_names_build_dict(active_lis_name_list, all_threads_list, "listen")
        report_failed_thread_names_assert_dict(radio_lis_thread_dict, listen_desired_types, "listen")


def report_failed_thread_names_assert_dict(radio_thread_dict, desired_thread_types, action):
    # compare the dict with the desired list

    for radio_name, thread_type_list in radio_thread_dict.items():
        for item in desired_thread_types:
            if item not in thread_type_list:
                message = f'{radio_name}: Error.{action}.Thread.{item} crashed.'
                ghetto.GNet.dict_error[radio_name] = message
                print(message)


def report_failed_thread_names_build_dict(name_list, all_threads_list, action):
    # fill a dict with {radio(n): ["tail", "head", "meta"]}
    rec_radio_thread_list_dict = {}
    for radio_name in name_list:
        thread_list = []
        rec_radio_thread_list_dict[radio_name] = thread_list
        for thread_name in all_threads_list:
            if radio_name in thread_name:
                thread_Type = thread_name[len(thread_name)-4:]
                thread_action = thread_name[len(thread_name)-11:len(thread_name)-5]
                if thread_action == action:
                    rec_radio_thread_list_dict[radio_name].append(thread_Type)

    return rec_radio_thread_list_dict


def report_active_radio_threads():
    """return True if (any) radio threads are active, write True to ghettoApi.radio_active

    shows an active internet connection, also during playing local files
    routes.delete_info() collects from eisenApi.radio_active for js
    """
    search_list = ["record", "listen"]
    for thread in threading.enumerate():
        for string in search_list:
            if string in thread.name:
                eisenApi.radio_active = True
                return True
    eisenApi.radio_active = False
    return False


def print_active_radio_threads():
    """ print active thread names for debug, also from GhettoRecorder
    """
    for thread in threading.enumerate():
        print(thread.name)


def scenario_kill_thread(name_action_type_tuple):
    """fill a tuple to kill a specific thread (thread ends itself on match)

    a_tuple = (name, action, type)
    ('classic', 'record', 'meta')
    type: tail, head, meta
    action: record   # listen
    name:
    test run can test error msg string in html input box
    test if recording works, if same name radio listen thread is dead
    test record writing if metadata crash
    """
    eisenTest.thread_killer = name_action_type_tuple


# ###### run scenario commands
# scenario_kill_thread(('br24', 'listen', 'meta'))
# scenario_kill_thread(('classic', 'record', 'tail'))



"""ghettorecorder console module
"""
from ghettorecorder import ghetto_recorder


def main():
    # command line version; write:python ghetto_recorder.py
    ghetto_recorder.terminal_main()


if __name__ == '__main__':
    main()



""" Waitress production server with random port assignment. """
import sys
import socket
import random
from os import path
from waitress import serve
from eisenradio import create_app

# flask wants its config
instance_dir = path.abspath(path.join(path.dirname(__file__), 'instance'))
sys.path.append(path.abspath(instance_dir))

port = random.randint(12488, 12974)    # random between two integers
print('\n Python WSGI "Waitress" serves flask\n')


def is_port_occupied(srv_port: int) -> bool:
    """The error indicator is 0 if the operation succeeded, otherwise return the value of the errno variable.
    """
    with socket.socket(socket.AF_INET, socket.SOCK_STREAM) as s:
        return s.connect_ex(('localhost', srv_port)) == 0


def main():
    global port
    while 1:
        if not is_port_occupied(port):
            app = create_app(port)  # transfer port num info to flask (for audio endpoint ...:5050/sound/starFM)
            serve(app, host='localhost', port=port)
            break
        else:
            print(f'Port in use: {port} \n\tadd one to port number')
            port += 1


if __name__ == "__main__":
    main()



"""
only alive for testing, since unit test do not init app.config.from_object('config.TestConfig')
writes an .env with test config and loads it
"""
from os import path, remove
from dotenv import load_dotenv

this_script_dir = path.dirname(__file__)


def write_config():
    global this_script_dir
    # get root of app, rip off one subdir .go up
    app_root = path.dirname(this_script_dir)
    radio_db_dir = path.join(app_root, 'app_writeable', 'db')

    # Secrets that actually belong to an encrypted vault; ci app encryption
    db_path_test = path.join(radio_db_dir, 'eisenradio_test.db')
    secret_num = '9fc2e5bd8372430fb6a1012af0b51f37'

    list_test = [
        'DATABASE=' + db_path_test,
        'FLASK_ENV=development',
        'DEBUG=True',
        'TESTING=True',
        'SECRET_KEY=' + secret_num
    ]

    remove_config()
    with open(path.join(this_script_dir, '.env'), 'w') as writer:
        for line in list_test:
            writer.write(line + '\n')
        writer.flush()

    load_config_os()

    # print(api.config)
    print('')
    remove_config()


def load_config_os():
    global this_script_dir
    load_dotenv(path.join(this_script_dir, '.env'))
    pass


def remove_config(new_path=None):
    global this_script_dir
    if new_path is not None:
        this_script_dir = new_path
    try:
        remove(path.join(this_script_dir, '.env'))
    except OSError:
        pass



import os
import threading
from time import sleep
from flask import jsonify, request, flash, redirect, url_for

from eisenradio.eisenhome import watchdog
from eisenradio.eisenutil import monitor_records, config_html
from eisenradio.lib.eisdb import get_db_connection, status_read_status_set, get_download_dir
from eisenradio.api import eisenApi
from ghettorecorder import ghettoApi
import ghettorecorder.ghetto_recorder as ghetto

first_run_index = True
first_run_audio_activated = False

status_listen_btn_dict = {}  # {table id 15: 1 } on
status_record_btn_dict = {}
radio_id_name_dict = {}
eisenApi.init_lis_btn_dict(status_listen_btn_dict)
eisenApi.init_rec_btn_dict(status_record_btn_dict)
eisenApi.init_radio_id_name_dict(radio_id_name_dict)
stop_blacklist_writer = False
ghettoApi.init_ghetto_stop_blacklist_writer(stop_blacklist_writer)
# the drop-down dialog of active recorder in console to jump to the recorder stations
active_streamer_dict = {}

last_btn_id_global = None  # current button pressed, or btn predecessor
combo_master_timer = 0
progress_master_percent = 0


def index_first_run(posts):
    """startup

    :params: posts: dump of table posts
    """
    global first_run_index
    if first_run_index:
        first_run_index = False
        check_write_protected()
        start_progress_timer_daemon()
        monitor_records.start_blacklist_writer_daemon()
        watchdog.start_watchdog_daemon()

        feed_status_btn_dicts(posts)
        feed_radio_id_name_dict(posts)
        config_html.tools_feature_settings_get_rows()


def feed_status_btn_dicts(posts):
    """reset all button tracker dicts to zero {table_id: button off}

    :params: posts: db dump of table posts
    """
    for row in posts:
        # init btn not pressed
        status_listen_btn_dict[row['id']] = 0
        status_record_btn_dict[row['id']] = 0


def feed_radio_id_name_dict(posts):
    """dict for resolving {id: radio_name} without open db connection everytime

    :params: posts: db dump of table posts
    """
    for row in posts:
        # api
        radio_id_name_dict[row['id']] = status_read_status_set(False, 'posts', 'title', row['id'])


def curr_radio_listen():
    """active radio

    :returns: tuple of two, active listening radio id and name for page refresh
    """
    current_station, current_id = "", ""
    for table_id, btn_down in status_listen_btn_dict.items():
        if btn_down:
            current_station = radio_id_name_dict[table_id]
            current_id = table_id
            break
    return current_station, current_id


def index_posts_clicked(post_request):
    """if rec return index_posts_record(),

    if listen return index_posts_clicked_already or return index_posts_clicked_new()
    first separation of action flow for rec and listen
    return False, deactivate btn of a newly created radio to introduce no source of error
    simplify reading, got either clicked new or already clicked listen btn
    """
    table_id = post_request['table_id']
    action = post_request['action']
    message = 'app restart required to use a new radio'

    if action == 'Record':
        if int(table_id) not in status_record_btn_dict.keys():
            print(message)
            return False

        return index_posts_record(table_id)

    if action == 'Listen':
        if int(table_id) not in status_listen_btn_dict.keys():
            print(message)
            return False

        radio_name = status_read_status_set(False, 'posts', 'title', table_id)
        if status_listen_btn_dict[int(table_id)]:  # 1, hello auto clicker; or on/off press
            return index_posts_clicked_already(table_id, radio_name)
        else:
            return index_posts_clicked_new(table_id, radio_name)


def index_posts_clicked_new(table_id, radio_name):
    """set global var last_btn_id_global, activate js auto clicker, announce sound endpoint for radio name

    | fresh listen button press
    | activate auto clicker to reset abandoned btn status (color)
    | JavaScript module gets sound endpoint url with current port number for html audio element
    | vars
    | 'sound_endpoint': "http://localhost:" + eisenApi.work_port + "/sound/"  - js audio connect to port number url
    """
    global last_btn_id_global

    status_listen_btn_dict[int(table_id)] = 1  # button down
    dispatch_master(int(table_id), 'Listen', 1)

    # last_btn_id_global init None, first radio not trigger click
    btn_to_switch = last_btn_id_global  # feed js auto clicker
    button_id = 'Listen_' + btn_to_switch if btn_to_switch is not None else None

    last_btn_id_global = table_id
    return jsonify(
        {'result': 'activate_audio',
         'button_to_switch': button_id,
         'radio_name': radio_name,
         'radio_id': table_id,
         'sound_endpoint': "http://localhost:" + str(eisenApi.work_port) + "/sound/"
         })


def index_posts_clicked_already(table_id, radio_name):
    """if js auto clicker send table_id for radio;
    if self pressed on/off, fill global var last_btn_id_global and deactivate audio --> better store it in api

    This thing here is about pressed LISTEN buttons.
    If listen button pressed, java auto clicker event is possible and change the button type (color).
    Auto clicker launches the whole event $("button").click(function () {} from $(document).ready(function () {
    AFTER the original button was pressed.
    Two scenarios for a 1 in status_listen_btn_dict[int(table_id)]:
    1) auto clicker
    2) same listen button was pressed on/off (twice)

    autoclicker vars
    'autoClicker': table_id
    self pressed btn vars
    'result': 'deactivate_audio',
    'radio_name': radio_name,
    'radio_id': "noId"         // tell all functions there is no listener active,
    'last_listen_id': table_id // but the style function must run to create a (minimal) record style, if active
    """
    global last_btn_id_global

    status_listen_btn_dict[int(table_id)] = 0
    dispatch_master(int(table_id), 'Listen', 0)
    # to make it clear 'button_to_switch' auto clicker var from index_posts_clicked_new is for NEW btn down evt
    if not last_btn_id_global == table_id:  # js press abandoned btn, not the same btn id, call style funct to reset var
        return jsonify({'autoClicker': table_id})    # to enable only minimal display if record in on
    else:
        last_btn_id_global = None  # same id, twice pressed, reset var
        return jsonify(
            {'result': 'deactivate_audio',
             'radio_name': radio_name,
             'radio_id': "noId",
             'last_listen_id': table_id
             })


def index_posts_record(table_id):
    """ call dispatch_master( id record on/off)
     | return names and table ids of all streamer to js
     | and id of current activated/deactivated rec
     | make a combo box with anchor to jump to the rec radio from console
     | vars
     | 'streamer': json_streamer - write name to the stream watcher drop-down dialog in console
    """
    conn = get_db_connection()
    radio_name = status_read_status_set(False, 'posts', 'title', table_id)
    conn.close()

    if status_record_btn_dict[int(table_id)]:
        del active_streamer_dict[radio_name]
        status_record_btn_dict[int(table_id)] = 0
        dispatch_master(int(table_id), 'Record', 0)
    else:
        active_streamer_dict[radio_name] = str(table_id)
        status_record_btn_dict[int(table_id)] = 1
        dispatch_master(int(table_id), 'Record', 1)

    # make combo box with anchor jumper from active_streamer_dict
    json_streamer = ''
    for streamer_name, streamer_id in active_streamer_dict.items():
        json_streamer = json_streamer + str(streamer_name) + '=' + str(streamer_id) + ','
    if not json_streamer:
        json_streamer = str('empty_json')

    return jsonify({'streamer': json_streamer, 'streamerId': table_id})


def query_radio_url(table_id):
    """return streaming url of radio"""
    radio_url = status_read_status_set(False, 'posts', 'content', table_id)
    return radio_url


def set_radio_path(table_id):
    """overwrite default path of ghetto_recorder.GBase.radio_base_dir with
    parent download_path
    create child dir with radio name

    GhettoRecorder stores streams by default under the current directory of settings.ini
    """
    parent_dir = status_read_status_set(False, 'posts', 'download_path', table_id)
    radio_name = status_read_status_set(False, 'posts', 'title', table_id)
    ghetto.GBase.radio_base_dir = parent_dir
    child_dir = os.path.join(parent_dir, radio_name)
    ghetto.GBase.make_directory(child_dir)


def dispatch_master(table_id, button, btn_status):
    """divide action for listen and rec, returns nothing

    prepare writing to dicts with radio name as key
    btn_status is 0 or 1, off or on
    """
    set_radio_path(table_id)
    radio_name = status_read_status_set(False, 'posts', 'title', table_id)

    if button == 'Listen':
        dispatch_listen_btn(radio_name, table_id, btn_status)

    if button == 'Record':
        dispatch_record_btn(radio_name, table_id, btn_status)


def dispatch_listen_btn(radio_name, table_id, lis_btn_pressed):
    """calls dispatch_record_start() if btn pressed, del from active dict if btn released, returns nothing

    write into listen activ dict if listen btn down or not
    tells the recorder dispatcher to start threads with listen action string
    """
    if lis_btn_pressed:
        ghetto.GRecorder.listen_active_dict[radio_name] = True
        if not status_record_btn_dict[table_id]:  # rec btn not pressed
            ghetto.GRecorder.record_active_dict[radio_name] = False  # rec not active
        dispatch_record_start(table_id, 'listen')

    if not lis_btn_pressed:
        ghetto.GRecorder.listen_active_dict[radio_name] = False


def dispatch_record_btn(radio_name, table_id, rec_btn_pressed):
    """calls dispatch_record_start() if btn pressed, del from active dict if btn released, returns nothing

    write into record_active_dict if radio shall be active rec or not
    check if listen btn is down and writes status to listen active dict
    recorder dispatcher shall start threads with record action string
    """
    if rec_btn_pressed:
        ghetto.GRecorder.record_active_dict[radio_name] = True
        if not status_listen_btn_dict[table_id]:  # listen btn not pressed
            ghetto.GRecorder.listen_active_dict[radio_name] = False

        dispatch_record_start(table_id, 'record')

    if not rec_btn_pressed:
        ghetto.GRecorder.record_active_dict[radio_name] = False  # rec thread stops, parameter action 'record'


def dispatch_record_start(table_id, action):
    """call dispatch_record_is_alive(), if alive check is playlist & start radio else write error dict, returns nothing

    test if server is alive, writes dict_error entry if url had failed
    if target server address is a playlist url m3u or pls,
    must read the playlist url first and extract radio url
    start threads with either listen or record action string to stop 'em separate if both listen and rec are active
    """
    radio_name = status_read_status_set(False, 'posts', 'title', table_id)
    radio_url = status_read_status_set(False, 'posts', 'content', table_id)

    playlist_url = dispatch_record_is_alive(radio_name, radio_url)
    if playlist_url is not None:
        radio_url = playlist_url

    if radio_name not in ghetto.GNet.dict_error.keys():
        ghetto.record(radio_name, radio_url, action)
    else:
        print(ghetto.GNet.dict_error[radio_name])
        ghetto.GBase.dict_exit[radio_name] = True  # end all threads of a failed radio; if any


def dispatch_record_is_alive(radio_name, radio_url):
    """call is_radio_online(), return url of first server in playlist, or return None

    test if server is alive, if not writes in error dict, later display msg in html
    if url is a playlist, it grabs the first url from playlist, mostly the best quality
    https://streams.br.de/bayern1obb_2.m3u
    """
    radio_url = ghetto.is_radio_online(radio_name, radio_url)
    if radio_url:
        return radio_url
    return


def start_progress_timer_daemon():
    """run progress_timer()"""
    threading.Thread(name="progress_timer", target=progress_timer, daemon=True).start()


def tear_down_after_timer_hit():
    """clean up to not confuse (buttons pressed, show online ...) if return to start page after timer hit"""
    ghettoApi.stop_blacklist_writer = True
    if len(ghetto.GBase.dict_exit) > 0:
        for recorder in ghetto.GBase.dict_exit:
            ghetto.GBase.dict_exit[recorder] = True
    for table_id in status_listen_btn_dict.keys():
        status_listen_btn_dict[table_id] = 0
    for table_id in status_record_btn_dict.keys():
        status_record_btn_dict[table_id] = 0
    active_streamer_dict.clear()


def progress_timer():
    """calculate remaining time until full stop of app, returns nothing

    stops ALL radio threads by writing ghetto.GBase.dict_exit[radio name] = True
    combo_master_timer -- val from browser in hours (default 0), -1 stops def, last loop
    progress_master_percent -- calculates percent of passed seconds (default 0)

    feed progress_master_percent global for route endpoint response to ajax updateMasterProgress()
    """
    global combo_master_timer  # combo in Tk/Tcl for drop-down dialog on ghetto_recorder package, the first GUI
    global progress_master_percent  # separate for future use; go to single timer for each radio

    up_count_timer = 0
    while 1:
        if combo_master_timer:
            if int(combo_master_timer) <= -1:
                seconds_to_run = 1  # must get more than 100 in percent calc, may not divide by zero; init exit
            else:
                seconds_to_run = (int(combo_master_timer) * 60 * 60)  # * 60
            percent = progress_bar_percent(up_count_timer, seconds_to_run)
            if percent:
                progress_master_percent = percent
            else:
                progress_master_percent = 0
                up_count_timer = 0

            if percent >= 100:
                tear_down_after_timer_hit()

        if not combo_master_timer:
            up_count_timer = 0
            progress_master_percent = 0

        up_count_timer += 1
        sleep(1)


def progress_bar_percent(up_count_timer, seconds_to_run):
    """calculate percent from passed by seconds and seconds to run, return float percent value

    return if zero is argument, because of division
    float for pytest type, since it could be sometimes int
    """
    if not seconds_to_run:
        return False
    # doing some math, p = (P * 100) / G, percent = (math.percentage value * 100) / base
    cur_percent = round((up_count_timer * 100) / seconds_to_run, 4)  # 0,0001 for 24h reaction to show in html
    return float(cur_percent)


def print_request_values(values):
    """helper to get all possible info printed from request"""
    for val in values:
        print(' -- start print --')
        print(f'\tval in request.form.values(): {val}')
        print(f'\trequest.data {request.data}')
        print(f'\trequest.form {request.form}')
        print(f'\trequest.values {request.values}')
        print(f'\trequest.form.to_dict() {request.form.to_dict()}')
        print(' -- end print --')


def check_write_protected():
    """write test b"\x03".hex() = 03, flash message if ok or not, redirect to start page

    to inform user if download directory is not writeable
    """
    try:
        download_dir = os.path.abspath(get_download_dir())
    except TypeError:
        return
    if download_dir is None:
        flash('Can not write to folder! No folder specified.', 'danger')
        flash('Use SAVE option from menu.', 'warning')
        return redirect(url_for('eisenhome_bp.index'))
    if download_dir:
        write_file = download_dir + '/eisen_write_test'
        try:
            with open(write_file, 'wb') as record_file:
                record_file.write(b'\x03')
            os.remove(write_file)
        except OSError:  # master of desaster
            flash('Can not write to folder!.' + download_dir, 'danger')
            flash('Use SAVE option from menu.', 'warning')
            return redirect(url_for('eisenhome_bp.index'))
    if not download_dir:
        flash('Can not write to folder! no folder specified' + download_dir, 'danger')
        flash('Use SAVE option from menu.', 'warning')
        return redirect(url_for('eisenhome_bp.index'))



""" monitor_records module description

    Info
        read all database blacklists from all radios into 'all_blacklists_dict'
        radio writes new title in 'recorder_new_title_dict[radio]'
        blacklist_writer updates all_blacklists_dict and updates the blacklist of each radio in db

        Case
           normal behavior - recorder copies new file (delete existing file)
           monitored - recorder compares title with all_blacklists_dict[radio] list,
           copy new file only if title is not found in blacklist

    Feature
       route endpoints added to utils route to create an DB entry for blacklist feature, can check status from java
       ghettoApi got a variable to break the run_blacklist_writer() loop, if timer is set
       Tools menu got Export/Import options
       Tools Export/Import menus are displayed or hidden Html Div elements (bp_utils.js)
"""
import copy
import json
import threading
import time

import eisenradio.lib.eisdb as eisen_db
from ghettorecorder import ghettoApi


all_blacklists_dict = {}
ghettoApi.init_ghetto_all_blacklists_dict(all_blacklists_dict)


def start_blacklist_writer_daemon():
    """runs run_blacklist_writer() thread until the end of app"""
    threading.Thread(name="blacklist_writer", target=run_blacklist_writer, daemon=True).start()


def run_blacklist_writer():
    """loop read feature db status and reflect it in api var for recorders, update radios blacklists with new titles

    each radio has its own blacklist
    loop ends if timer on start page is set
    """
    first_run = True
    while not ghettoApi.stop_blacklist_writer:
        if first_run:
            enabled = status_db_blacklist_get()
            status_api_blacklist_set(enabled)
            feed_api_radios_blacklists()
            first_run = False

        if ghettoApi.blacklist_enable:
            update_radios_blacklists()

        for _ in range(5):
            if ghettoApi.stop_blacklist_writer:
                break
            time.sleep(1)


def status_db_blacklist_get():
    """return (0,1) off/on

    "misused" column, cause older db's don't have extra columns
    called by
    @eisenutil_bp.route('/tools'
    @eisenutil_bp.route('/tools_radio_blacklist_set'
    """
    conn = eisen_db.get_db_connection()
    status = conn.execute('SELECT browser_open FROM eisen_intern WHERE id = 2').fetchone()
    conn.close()
    if status:
        return status
    return


def status_api_blacklist_set(status):
    """switch status update api, return nothing"""
    if status is None:
        # not enabled at all, no insert added a new row 2
        ghettoApi.blacklist_enable = False
    if status is not None:
        if not int(status[0]):
            ghettoApi.blacklist_enable = False
        if int(status[0]):
            ghettoApi.blacklist_enable = True


def blacklist_enabled_button_outfit_get(is_enabled):
    """Read blacklist feature table row, return enabled or not.

    change btn color in Tools menu Monitor Records to reflect status
    called by @eisenutil_bp.route('/tools'
    """
    enabled = False
    if is_enabled is None:
        enabled = False
    if is_enabled:
        if not int(is_enabled[0]):
            enabled = False
        if int(is_enabled[0]):
            enabled = True
    return enabled


def update_radios_blacklists():
    """for loop to update blacklists for all currently active recorder

    pseudo sample
       recorder file dict -    recorder_new_title_dict['radio5'] = 'ASAP - I want to be part of the blacklist'
       recorder compares  -        all_blacklists_dict['radio5'] = ['The Listies - I am on a list', 'OMG - Mee Too']
       comparison
       blacklist_writer: if recorder_new_title_dict['radio5'] value NOT in all_blacklists_dict['radio5'] list
       then
       blacklist_writer: all_blacklists_dict['radio5'].append(recorder_new_title_dict['radio5'] value)
    """
    # make a copy of dict to prevent 'RuntimeError: dictionary changed size during iteration'
    recorder_new_cp = copy.deepcopy(ghettoApi.recorder_new_title_dict)

    for radio, new_title in recorder_new_cp.items():
        if new_title not in ghettoApi.all_blacklists_dict[radio]:
            conn = eisen_db.get_db_connection()
            blacklist = ghettoApi.all_blacklists_dict[radio]
            blacklist.append(new_title)
            conn.execute('UPDATE posts SET display = ? WHERE title = ?;', (json.dumps(blacklist), radio))
            conn.commit()
            conn.close()


def feed_api_radios_blacklists():
    """for each radio write blacklist from db to api for comparison

    recorder can check if record (title) is in the list and skip writing it
    """
    conn = eisen_db.get_db_connection()
    radios_list = conn.execute('SELECT title FROM posts').fetchall()

    if radios_list:
        for row in radios_list:
            radio = row[0]
            # table column 'display' was used to store current title for displaying, now in api; reuse for list
            known_files = conn.execute('SELECT display FROM posts WHERE title = ?;', (radio,)).fetchone()

            blacklist = json_loads_blacklist(known_files, radio)
            ghettoApi.all_blacklists_dict[radio] = blacklist
    conn.close()


def dump_radio_blacklist_from_col(radio):
    """call json_loads_blacklist() return blacklist as list"""
    conn = eisen_db.get_db_connection()
    known_files = conn.execute('SELECT display FROM posts WHERE title = ?;', (radio,)).fetchone()
    conn.close()

    blacklist = json_loads_blacklist(known_files, radio)
    return blacklist


def json_loads_blacklist(known_files, radio):
    """json.loads(sqlite row object) to list, return the blacklist, [] list on exception if json can not read db"""
    blacklist = []
    if known_files:
        if known_files[0]:
            try:
                blacklist = json.loads(known_files[0])
            except Exception as error:
                message = f'minor error in json_loads_blacklist(), {radio} {error}, can proceed with an empty list'
                print(message)
    return blacklist


def sort_dictionary_by_value(blacklist):
    """enum blacklist, sort output to tuples, make a sorted_dict from sorted tuples, return value sorted dict

    sorted_dict for display, route endpoint shows it in Html for editing and deletion
    sorted_dict to store index numbers of blacklist (list) to delete index numbers later without value comparison, to
    prevent a possible introduction of a source of error
    """
    sorted_dict = {}
    if not len(blacklist):
        return sorted_dict

    unsorted_dict = {index: value for index, value in enumerate(blacklist)}
    # Create a list of tuples sorted by value, value[1] is value field in dict/ value[0] mimics "key"
    sorted_tuples = sorted(unsorted_dict.items(), key=lambda value: value[1])
    sorted_dict = {elem[0]: elem[1] for elem in sorted_tuples}
    return sorted_dict


def delete_blacklist(radio):
    """delete a blacklist from db, returns nothing

    delete a radio blacklist from db and the all_blacklists_dict, via ajax request or delete_all_blacklists() call
    """
    message = f'replace blacklist {radio} with an empty list!! '
    lst = [message]
    print(message)
    blacklist = []
    conn = eisen_db.get_db_connection()
    conn.execute('UPDATE posts SET display = ? WHERE title = ?;', (json.dumps(blacklist), radio))
    conn.commit()
    conn.close()
    ghettoApi.all_blacklists_dict[radio] = []
    return lst


def del_single_title_master(radio, request_dict):
    """return log, take radio blacklist and sort indexes to delete, call deletion function, write updated blacklist

    del individual items from a radios blacklist, ajax delivers list of title indexes for deletion in request_dict
    avoid comparing values
    1 sort del indexes in ascending list,
    2 del a title and subtract one from original index in each loop
    sorted values in new ordered list; sort to later remove items via index in a foreseen manner
    """
    log_list = []
    blacklist = dump_radio_blacklist_from_col(radio)
    if not blacklist:
        return

    for item in blacklist:
        message = f'idx^{blacklist.index(item)}^ {item}'
        print(message)
        log_list.append(message)

    message = "\n --> Delete titles from blacklist \n"
    print(message)

    unsorted_del_idx = del_single_title_get_del_indexes(request_dict)
    sorted_del_idx = sorted(unsorted_del_idx)

    altered_blacklist, log_idx_list = del_single_title_del_indexes(sorted_del_idx, blacklist, radio)
    del_single_title_write(altered_blacklist, radio)

    log_idx_list.extend(log_list)    # reverse to have the del idx at first and mark 'em in the list
    return log_idx_list


def del_single_title_write(altered_blacklist, radio):
    """json.dumps(altered_blacklist), to the radio in db, returns nothing"""
    conn = eisen_db.get_db_connection()
    conn.execute('UPDATE posts SET display = ? WHERE title = ?;', (json.dumps(altered_blacklist), radio))
    conn.commit()


def del_single_title_del_indexes(asc_ord_del_idx, blacklist, radio):
    """list of indexes to delete from a radio blacklist, return the updated blacklist

    remove title(s) from list by deleting their indexes, no value compare
    list with indexes to delete is ordered ascending [7,23,467] (same as the blacklist[0] to blacklist[999])
    index deletion means -1 index number for all successors of the deleted element vs org. list
    returns updated blacklist
    """
    log_list = ["--> index delete:"]
    print(asc_ord_del_idx)

    for count, idx in enumerate(asc_ord_del_idx):
        message = f'index:{idx}: {blacklist[idx + count]} [{idx + count}: index in altered list]'
        print(message)
        log_list.append(message)
        del ghettoApi.all_blacklists_dict[radio][idx + count]
        del blacklist[idx + count]

    log_list.append("--> from this list:")
    return blacklist, log_list


def del_single_title_get_del_indexes(request_dict):
    """return the extracted, unsorted list with indexes to delete

    the request dict transports not only indexes via ajax
    pull del indexes from request
    """
    unsorted_indexes = []

    for key, value in request_dict.items():
        if (not key == 'radio_name') and (not key == 'delAll'):
            unsorted_indexes.append(int(value))
    return unsorted_indexes


def delete_all_blacklists(feature_enabled):
    """delete blacklist of each radio if feature was disabled in 'Tools Monitor Records', returns nothing

    caller @eisenutil_bp.route('/radio_blacklist_set'
    delete all collected filenames from all radios if feature switch to disabled
    """
    if not feature_enabled:
        conn = eisen_db.get_db_connection()
        radios = conn.execute('SELECT title FROM posts;').fetchall()
        conn.close()

        if radios is not None:
            for row in radios:
                radio = row[0]
                delete_blacklist(radio)


def feature_blacklist_switch_status(table_row):
    """enables feature if status is None (no db table cell) or switch status if bool, writes status, return new status

    first run ever creates db entry for the feature,
    then switch and returns on or off
    """
    enabled = False
    first_run = False
    if table_row is None:
        enabled = True
        ghettoApi.blacklist_enable = enabled
        first_run = True
    if table_row is not None:
        enabled = blacklist_enable(table_row)

    if first_run:
        feature_blacklist_insert_row(enabled)
    if not first_run:
        feature_blacklist_update_row(enabled)
    return enabled


def blacklist_enable(table_row):
    blacklist_on = int(table_row[0])
    enabled = False
    if not blacklist_on:
        enabled = True
        ghettoApi.blacklist_enable = enabled
    if blacklist_on:
        enabled = False
        ghettoApi.blacklist_enable = enabled
    return enabled


def feature_blacklist_insert_row(enabled):
    conn = eisen_db.get_db_connection()
    conn.execute('INSERT INTO eisen_intern (browser_open) VALUES (?);', (enabled,))
    conn.commit()
    conn.close()


def feature_blacklist_update_row(enabled):
    conn = eisen_db.get_db_connection()
    conn.execute('UPDATE eisen_intern SET browser_open = ? WHERE id = ?;', (enabled, 2))
    conn.commit()
    conn.close()


def fill_radio_display_col():
    """mocking the db entries for dev and test"""

    # call it in eishome.index_first_run()
    classic = ['Aurèle Marthan Rafaël Angster Guillaume Begni Philibert Perrine & Amaury Viduvier - Quintet in E-Flat '
               'Major K 452 III Allegretto', 'Vladimir Ashkenazy - Nocturne No 20 in C sharp Minor Op posth']
    goa_psi = ['Electric Universe - Activate  animaskntru', 'Bafoomay - Monosonicum Miles Away Mix  animaskntru']
    time_machine = ['Murder at Midnight - 12 The Man Who Died Yesterday', "The Adventures of Philip Marlowe - The "
                                                                          "Fox's Tail"]
    nerd_sound_tracks = ['Lyn - Autonomy', 'Takayuki Iwai Yuki Iwai Isao Abe Hideki Okugawa Tetsuya Shibata - Active '
                                           'Red Theme of Ken']  # nerd-sound-tracks
    BLUES_UK = ['Nothing But The Devil', 'Henrik - Schlader band']
    relax = ['Mission Brown - Another Beginning', 'Neil Davidge - Sensor Melo Remix']
    Nachtflug = ['Helium Vola - Selig', 'Kite - Hand Out the Drugs']
    hm = ['E-Spectro - Dawn On Sunset Vla DSound Rem', "Alex O'Rion - Komodo"]
    Boradio = ['AutoDJ - Various']
    B2 = ['Hör auf Dein Herz wwwSchlagerRadiode', 'Beatrice Egli - Verrückt Nach Dir']

    radio_dict = {
        'classic': classic,
        'goa_psi': goa_psi,
        'time_machine': time_machine,
        'nerd-sound-tracks': nerd_sound_tracks,
        'BLUES_UK': BLUES_UK,
        'relax': relax,
        'Nachtflug': Nachtflug,
        'hm': hm,
        'Boradio': Boradio,
        'B2': B2
    }

    conn = eisen_db.get_db_connection()

    for radio, blacklist in radio_dict.items():
        # json.dumps() to string, json.loads() to object
        conn.execute('UPDATE posts SET display = ? WHERE title = ?;', (json.dumps(blacklist), radio))
        conn.commit()

    conn.close()



from flask import Blueprint, render_template, request, flash, redirect, url_for, make_response, jsonify
from eisenradio.eisenutil import config_html
import eisenradio.eisenhome.eishome as eis_home
from eisenradio.lib.eisdb import get_post, delete_radio, enum_radios, get_db_connection
from ghettorecorder import ghettoApi
from eisenradio.api import eisenApi

# Blueprint Configuration
eisenhome_bp = Blueprint(
    'eisenhome_bp', __name__,
    template_folder='bp_home_templates',
    static_folder='bp_home_static',
    static_url_path='/bp_home_static'
)


@eisenhome_bp.route('/', methods=('GET', 'POST'))
def index():
    """draws start page

    dump db and run eis_home.index_first_run() to prepare app, fill dicts, start daemons
    collect vars for jinja to draw start page from template

    POST: js sends post with button click event listener, / index endpoint return json dict to js
    return: vars for looping over the html elements and provide names of radios, button color, title for console

    * posts - db dump
    * combo_master_timer - actual timer value call eis_home.combo_master_timer (default 0)
    * status_listen_btn_dict - whole dict eis_home.status_listen_btn_dict to set button colors if pressed (default 0)
    * status_record_btn_dict - whole dict eis_home.status_record_btn_dict to set button colors if pressed (default 0)
    * current_station - write name of listened station to console if any (default None)
    * current_table_id - table id of listened station (default None)
    * listen_last_url - feed the audio element with url after page refresh (default None)
    """
    local_host_sound_route = "http://localhost:" + str(eisenApi.work_port) + "/sound/"
    listen_last_url = ""

    posts = enum_radios()
    eis_home.index_first_run(posts)
    # first call returns None
    current_station, current_table_id = eis_home.curr_radio_listen()

    if request.method == 'POST':
        # print_request_values(request.form.values())
        post_request = request.form.to_dict()  # flat dict werkzeug doc
        # $("button").click(function () {}, ajax /
        json_post = eis_home.index_posts_clicked(post_request)
        # returns False on error to deactivate buttons for newly created radios, app restart required
        if json_post:
            return json_post

    # for page reload, connect to last sound endpoint again
    if current_station:
        listen_last_url = local_host_sound_route + current_station

    return render_template('bp_home_index.html',
                           posts=posts,

                           combo_master_timer=eis_home.combo_master_timer,
                           status_listen_btn_dict=eis_home.status_listen_btn_dict,
                           status_record_btn_dict=eis_home.status_record_btn_dict,
                           current_station=current_station,
                           current_table_id=current_table_id,
                           listen_last_url=listen_last_url)


@eisenhome_bp.route('/degrade_animation_level_get', methods=['GET'])
def degrade_animation_level_get():
    """ return the animation level status for CPU utilisation from db, high is 1, low is 0,
    also called on js onLoad to fill the animationsAllowedDict
    db return value is a sql row object, one must choose the column
    jsonify first key is name of the caller function in js to separate (follow) ajax return values better
    """
    row_html_cpu = 8
    conn = get_db_connection()
    on = conn.execute('SELECT browser_open FROM eisen_intern WHERE id =' + str(row_html_cpu) + ';').fetchone()
    conn.close()
    cpu_utilization = "high" if on[0] else "low"
    return jsonify({"degradeAnimationsWriteDict": cpu_utilization})


@eisenhome_bp.route('/degrade_animation_level_set', methods=['POST'])
def degrade_animation_level_set():
    """ degrade_animation_level_set() and degrade_animation_level_get() called on radio button change to
    write the js animationsAllowedDict new, so animation can see if run is allowed

    | write the animation level status for CPU to db, high is 1 (default in db), low is 0
    | We can only use 1 and 0 values since db writer uses on/off style, default is 1.

    | config_html.tools_feature_settings_get_rows() compares actual rows number in db with rows hardcoded as a list
    | it calls tools_feature_settings_create_default(feature_without_row) to create the rows with default value
    | config_html.tools_feature_settings_get_rows is called on startup to write new rows if version upgrade of package
    | database row 8 for html_cpu
    """
    on = 1
    off = 0
    row_html_cpu = 8
    conn = get_db_connection()
    status = conn.execute('SELECT browser_open FROM eisen_intern WHERE id =' + str(row_html_cpu) + ';').fetchone()
    conn.close()
    toggle = off if status[0] else on
    config_html.tools_feature_toggle_show_html_config(toggle, row_html_cpu)
    return jsonify({"degradeAnimationsSet": "Elvis lebt!"})


@eisenhome_bp.route('/tools_radio_config_get', methods=['GET'])
def tools_radio_config_get():
    """ return a dictionary with HTML settings, tell the js how to style the app: style, animations;

    config_html.tools_feature_settings_get_rows() has all rows hardcoded as a list
    """
    row_db_html_animation = 3
    row_db_html_style = 4
    row_db_html_front_pigs = 5
    row_db_html_balloon = 6
    row_db_html_speaker = 7
    row_db_html_cpu = 8

    row_list = [
        row_db_html_animation,
        row_db_html_style,
        row_db_html_front_pigs,
        row_db_html_balloon,
        row_db_html_speaker,
        row_db_html_cpu,
    ]
    feature_list = [
        "checkboxConfigAnimation",
        "checkboxConfigStyle",
        "checkboxConfigFrontPigs",
        "checkboxConfigBalloon",
        "checkboxConfigFlatSpeaker",
        "cpuUtilisation",
    ]
    check_box_status_list = list(map(lambda row: config_html.tools_feature_on_off_state_in_db(row), row_list))
    feature_checked_tuple = zip(feature_list, check_box_status_list)
    settings_dict = {feature: checked for feature, checked in feature_checked_tuple}

    return jsonify({"configEisenradioHtmlSetting": settings_dict})


@eisenhome_bp.route('/page_flash', methods=('GET', 'POST'))
def page_flash():
    """reset timer vars, return flash info on info page"""
    eis_home.combo_master_timer = 0  # master timer recording
    eis_home.progress_master_percent = 0

    flash('Count down timer ended all activities. App restart recommended!', 'success')
    return render_template('bp_home_page_flash.html')


@eisenhome_bp.route('/<int:id>/delete', methods=['POST'])
def delete(id):
    """delete a radio only if it is not active, return a flash of ok or not, redirect to start page"""
    if eis_home.status_listen_btn_dict[id] or eis_home.status_record_btn_dict[id]:
        flash('Radio is active. No deletion.', 'warning')
        return redirect(url_for('eisenhome_bp.index'))

    post = get_post(id)
    rv = delete_radio(id)
    if rv:
        flash('"{}" was successfully deleted!'.format(post['title']), 'success')
    if not rv:
        flash('"{}" was not deleted!'.format(post['title']), 'warning')
    return redirect(url_for('eisenhome_bp.index'))


@eisenhome_bp.route('/cookie_set_dark', methods=['GET', 'POST'])
def cookie_set_dark():
    """darkmode cookie

    secure=False for http and https
    localhost is an inner Network of the computer used by Eisenradio server and browser, therefore a save place
    secure=True without external signed ssl certs will NOT rise security nor let disappear dev console errors in IEx
    """
    resp = make_response("Eisenkekse sind die besten")
    resp.set_cookie('eisen-cookie', 'darkmode', max_age=60 * 60 * 24 * 365 * 2, secure=False, httponly=True)
    return resp


@eisenhome_bp.route('/cookie_get_dark', methods=['GET'])
def cookie_get_dark():
    """return color style of startpage AND id of listener button if any"""
    listener_id = None
    for radio, listening in ghettoApi.listen_active_dict.items():
        if listening:
            for r_id, name in eisenApi.radio_id_name_dict.items():
                if name == radio:
                    listener_id = r_id
                    break
    mode = request.cookies.get('eisen-cookie', None)
    return jsonify({"darkmode": mode,
                    "listenerId": listener_id
                    })


@eisenhome_bp.route('/cookie_del_dark', methods=['POST'])
def cookie_del_dark():
    resp = make_response("necesito nuevas cookies")
    resp.set_cookie('eisen-cookie', max_age=0)
    return resp


@eisenhome_bp.route('/station_get', methods=['GET'])
def station_get():
    """return json dict {radio: id}

    curr_radio_listen() get button down name and id
    rebuild console after page refresh
    empty dict if no listen, else send name and db table id
    """
    listen_dict = {}
    current_station, current_table_id = eis_home.curr_radio_listen()
    if len(current_station) > 0:
        listen_dict[current_station] = current_table_id
    return jsonify({'stationGet': listen_dict})


@eisenhome_bp.route('/streamer_get', methods=['GET'])
def streamer_get():
    """return json dict of all active streams rec;

    populate dropdown in console after page refresh
    empty dict if no streamer, else send name and db table id
    """
    streamer_dict = {}
    if len(eis_home.active_streamer_dict) > 0:
        for rec_station, rec_station_id in eis_home.active_streamer_dict.items():
            streamer_dict[rec_station] = rec_station_id
    return jsonify({'streamerGet': streamer_dict})


@eisenhome_bp.route('/skipped_records_get', methods=['GET'])
def skipped_records_get():
    """return json list of all radios that skipped writing a record file due to the blacklist
    empty dict if nothing skipped
    """
    skipped_records_list = []
    eisenApi.init_radio_name_id_dict()
    eisenApi.init_skipped_record_eisen_dict()

    for radio in eisenApi.skipped_record_eisen_dict.keys():
        if eisenApi.get_skipped_record(radio):
            skipped_records_list.append(eisenApi.radio_name_id_dict[radio])

    return jsonify({'skippedRecordsGet': skipped_records_list})


@eisenhome_bp.route('/cookie_set_show_visuals', methods=['GET', 'POST'])
def cookie_set_show_visuals():
    """spectrum analyser cookie

    standard cookie
    """
    resp = make_response("disable visualisation")
    resp.set_cookie('eisen-cookie-visuals', 'show_visuals', max_age=60 * 60 * 24 * 365 * 2, secure=False, httponly=True)
    eisenApi.init_show_analyser(False)
    return resp


@eisenhome_bp.route('/cookie_get_show_visuals', methods=['GET'])
def cookie_get_show_visuals():
    rv = request.cookies.get('eisen-cookie-visuals', None)

    if rv == 'show_visuals':
        return jsonify({'str_visuals': 'show_visuals'})
    if not rv:
        return jsonify({'str_visuals': '-empty-'})


@eisenhome_bp.route('/cookie_del_show_visuals', methods=['POST'])
def cookie_del_show_visuals():
    resp = make_response("bye\neisen-cookie-visuals")
    resp.set_cookie('eisen-cookie-visuals', max_age=-1)
    eisenApi.init_show_analyser(True)
    return resp


@eisenhome_bp.route('/index_posts_combo', methods=['POST'])
def index_posts_combo():
    """set hours for timer, return read val for test"""
    eis_home.combo_master_timer = request.form['timeRecordSelectAll']
    return eis_home.combo_master_timer


@eisenhome_bp.route('/index_posts_percent', methods=['GET'])
def index_posts_percent():
    """return percent of timer to js"""
    return jsonify({'result': eis_home.progress_master_percent})


@eisenhome_bp.route('/display_info', methods=['GET'])
def display_info():
    """return a dict with radio {id: message},

    message is either the currently played title in the radio or an error message
    JS writes message to html element with radio id
    updateDisplay() ajax call
    """
    if request.method == "GET":
        id_text_dict = {}
        try:
            for radio_name, radio_text in ghettoApi.current_song_dict.items():
                for radio_db_id, radio_title in eisenApi.radio_id_name_dict.items():
                    if radio_name == radio_title:
                        if len(radio_text) > 0:
                            id_text_dict[str(radio_db_id)] = str(radio_text)

            for radio_name, radio_error in ghettoApi.ghetto_dict_error.items():
                for radio_db_id, radio_title in eisenApi.radio_id_name_dict.items():
                    if radio_name == radio_title:
                        id_text_dict[str(radio_db_id)] = str(radio_error)
        except Exception as error:
            print(f'display_info(): {error}')

        return jsonify({"updateDisplay": id_text_dict})


@eisenhome_bp.route('/all_radio_table_ids_and_names_get', methods=['GET'])
def all_radio_table_ids_and_names_get():
    """returns a dict with all radio {id: name, id2: name2},

    build js style instance for each radio
    """
    return jsonify({"eisenRadioCreateStyleInstances": eisenApi.radio_id_name_dict})



"""module description config_html

    target: remove front-end settings: style, animations for low-end devices
    - feature row id:
    1 browser autostart,
    2 blacklist,
    3 html animation,
    4 html style
    5 inflated animals
    6 animated balloon
    7 animated speaker
    8 cpu utilisation used for animation to switch of most animations on the fly
    - row 2 must be created first, since its existence is not sure
    - keep app usable on low-end devices (70 mb RAM used with two or three connections)
    - endpoint response is a log list to show user what is in the box

    each feature has a checkbox to toggle status
    tools_feature_settings_get_rows(): call on startup to write all rows on version upgrade, value 1 except blacklist 0
"""
import eisenradio.lib.eisdb as eisen_db

enabled = 1
disabled = 0

# names must match the list and represent database row id
row_browser_autostart = 1
row_blacklist = 2
row_html_animation = 3
row_html_style = 4
row_html_front_pigs = 5
row_html_balloon = 6
row_html_speaker = 7
row_html_cpu = 8

feature_list = [
    row_browser_autostart,
    row_blacklist,
    row_html_animation,
    row_html_style,
    row_html_front_pigs,
    row_html_balloon,
    row_html_speaker,
    row_html_cpu,
]


def tools_feature_status_in_db(row_number):
    """return sql row object if exists or none / workaround if status[0] is 0, false """
    conn = eisen_db.get_db_connection()
    status = conn.execute('SELECT browser_open FROM eisen_intern WHERE id =' + str(row_number) + ';').fetchone()
    conn.close()
    if status:
        return status
    return


def tools_feature_on_off_state_in_db(row_number):
    """return (0,1) off/on or none"""
    conn = eisen_db.get_db_connection()
    content = conn.execute('SELECT browser_open FROM eisen_intern WHERE id =' + str(row_number) + ';').fetchone()
    conn.close()
    if content:
        return content[0]
    return


def tools_feature_settings_create_default(feature_without_row):
    """ set all features to ON, except blacklist """
    conn = eisen_db.get_db_connection()
    for row_number in feature_without_row:
        default_value = 0 if row_number == row_blacklist else 1
        conn.execute('INSERT INTO eisen_intern (id, browser_open) VALUES (?,?);', (row_number, default_value))
    conn.commit()
    conn.close()


def tools_feature_settings_get_rows():
    """ called on startup to write all rows on version upgrade
    row id 4 val 1, row id 5 val 0 and so on
    """
    feature_without_row = [row_num for row_num in feature_list if not tools_feature_status_in_db(str(row_num))]
    if len(feature_without_row) > 0:
        tools_feature_settings_create_default(feature_without_row)


def tools_feature_toggle_show_html_config(status, row):
    """ toggle html settings """
    is_on = 1 if status else 0
    conn = eisen_db.get_db_connection()
    conn.execute('UPDATE eisen_intern SET browser_open = ? WHERE id = ?;', (is_on, str(row)))
    conn.commit()
    conn.close()
    return is_on



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="eisenradio",
    version="2.3.3",
    author="René Horn",
    author_email="rene_horn@gmx.net",
    description="Play radio. Style your App.",
    long_description=long_description,
    license='MIT',
    long_description_content_type="text/markdown",
    url="https://github.com/44xtc44",
    include_package_data=True,
    packages=setuptools.find_packages(),
        install_requires=[
            'aacrepair',
            'ghettorecorder==2.4.7',
            'flask>=2.3.2',
            'Werkzeug>=2.2.3',
            'certifi',
            'configparser~=5.0.2',
            'pytest~=6.2.5',
            'python-dotenv~=0.19.2',
            'waitress',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable",
        "Topic :: Multimedia :: Sound/Audio :: Capture/Recording",
    ],
    python_requires=">=3.8",
)



"""
Flask configuration Not in the version control
"""
import os

this_script_dir = os.path.dirname(__file__)
app_root = os. path.dirname(this_script_dir)
radio_db_dir = os.path.join(app_root, 'app_writeable', 'db')

is_snap_device = 'SNAP' in os.environ
is_android_device = 'ANDROID_STORAGE' in os.environ


class Config:
    """Base config."""
    SECRET_KEY = "9fc2e5bd8372430fb6a1012af0b51f37"


class ProdConfig(Config):
    FLASK_ENV = 'production'
    DEBUG = False
    TESTING = False
    DATABASE_URI = os.path.join(radio_db_dir, 'database.db')
    DATABASE = os.path.join(radio_db_dir, 'database.db')
    PYTEST = False


class AndroidConfig(Config):
    FLASK_ENV = 'production'
    DEBUG = False
    TESTING = False
    if is_android_device:
        DATABASE_URI = "/storage/emulated/0/Music/EisenRadio/eisenradio_android.db"
        DATABASE = "/storage/emulated/0/Music/EisenRadio/eisenradio_android.db"
    ANDROID_DEVICE = True
    PYTEST = False


class SnapConfig(Config):
    FLASK_ENV = 'production'
    DEBUG = False
    TESTING = False
    if is_snap_device:
        DATABASE_URI = os.environ["SNAP_USER_COMMON"] + '//database.db'
        DATABASE = os.environ["SNAP_USER_COMMON"] + '//database.db'
    PYTEST = False


class TestConfig(Config):
    """pep 257 senseless class because unit test can not convince flask to load it, I give up here

    load from inside flask works; use old school config_apfac.py to test it, write an .env
    """
    FLASK_ENV = 'development'
    DEBUG = True
    TESTING = True
    # DATABASE_URI = os.path.join(radio_db_dir, 'eisenradio_test.db')
    DATABASE = os.path.join(radio_db_dir, 'eisenradio_test.db')
    PYTEST = True
